# -*- encoding: utf-8 -*-

import datetime
from datetime import timedelta
import urllib, urllib2
import simplejson
import pkg_resources
from  pkg_resources import parse_version
from django.db import models
from django.utils.translation import ugettext as _
from setuptools.package_index import PackageIndex as STPackageIndex
from setuptools.package_index import open_with_auth, local_open
from pkg_resources import parse_version


class PackageIndex(STPackageIndex):
    """
    Override to provide a wat to deal with connection problem.
    """
    def open_url(self, url, warning=None):
        if url.startswith('file:'): return local_open(url)
        try:
            return open_with_auth(url)
        except urllib2.HTTPError, v:
          pass
        except urllib2.URLError, v:
          print url


CACHE_HOURS_TIME = 24

class IndexServer(models.Model):
    """
    Index Server package where package will be searched.
    """
    id = models.AutoField(primary_key=True)
    url = models.CharField(max_length=255, verbose_name=_('URL'))
    name = models.CharField(max_length=255, verbose_name=_('Server name'))
    
    def __unicode__(self, ):
        return "%s (%s)" % (self.name, self.url, )

class VersionCache(models.Model):
    """
    Cache lastest version per package.
    """
    id = models.AutoField(primary_key=True)
    package = models.CharField(max_length=255, verbose_name=_('Package name'))
    version = models.CharField(max_length=255, verbose_name=_('Version'))
    last_check = models.DateTimeField(auto_now=True)

class MonitoredSite(models.Model):
    """
    Represent website which will be present to check its versions.
    It has to provide ones via the agent.
    """
    id = models.AutoField(primary_key=True)
    url = models.CharField(max_length=255, verbose_name=_('URL') )
    name = models.CharField(max_length=255, verbose_name=_('Server name'))
    token = models.CharField(max_length=255, verbose_name=_('Token'))

    class Meta:
        verbose_name = _("Site")

    def get_packages_versions(self,):
        """
        Ask to the remote site via HTTP, its versions packages, dict is returned.
        """
        request_params = urllib.urlencode({'token': self.token, })
        safe_url = "%s/" % self.url.strip("/")
        page = urllib2.urlopen(safe_url, request_params)
        json_page_versions = page.read()
        dict_versions = simplejson.loads(json_page_versions)
        return dict_versions

    def get_last_package_version(self, pkg, ):
        """
        Check for the last for on each server index, the greater is returned, else None.
        There is a Db cache system, to store the lastest version for each package name.
        """
        vc, created = VersionCache.objects.get_or_create(package=pkg)
        if vc.version != "" and (vc.last_check + timedelta(hours=CACHE_HOURS_TIME)) > datetime.datetime.now():
            return vc.version
        
        versions = []
        for index in IndexServer.objects.all():

          pypi = PackageIndex(index_url=index.url)
          pypi.scan_all()
          req = pkg_resources.Requirement.parse(pkg)
          dist = pypi.obtain(req)

          if dist is not None:
              versions.append(dist.version)

        versions.sort(key=lambda x: parse_version(x))
        if len(versions) > 0:
            vc.version = versions[-1]
            vc.last_check = datetime.datetime.now()
            vc.save()
            return versions[-1]
        else:
            return None


    def check_packages_versions(self, ):
        """
        Compare each package with the current on remote site and the latest
        available. Package dict is return with ones which are not up-to-date.
        """
        dict_versions = {}
        for pkg, versions in self.get_packages_versions().items():
            current_version = versions[0]

            last_version = self.get_last_package_version(pkg)
            if last_version is None:
                dict_versions[pkg] = ["????", ]
                continue

            if parse_version(last_version) > parse_version(current_version):
                if not dict_versions.has_key(pkg):
                    dict_versions[pkg] = []
                dict_versions[pkg].append(last_version)
        return dict_versions


    def __unicode__(self, ):
        return "%s (%s)" % (self.name, self.url, )
